﻿using System;
using System.IO;
using System.Linq;
using Google.Contacts;
using Google.GData.Contacts;
using Google.GData.Extensions;
using Outlook = Microsoft.Office.Interop.Outlook;

namespace Palmmedia.GoogleSync.Synchronization
{
    /// <summary>
    /// Creates Outlook contacts from Google contacts.
    /// </summary>
    internal static class ContactBuilder
    {
        /// <summary>
        /// Creates an Outlook contact.
        /// </summary>
        /// <param name="application">The application.</param>
        /// <param name="source">The Google contact.</param>
        /// <returns>The Outlook contact.</returns>
        public static Outlook.ContactItem CreateContact(Outlook.Application application, ContactWrapper source)
        {
            var contactItem = application.CreateItem(Outlook.OlItemType.olContactItem) as Outlook.ContactItem;

            SetName(contactItem, source.Contact);
            SetAddresses(contactItem, source.Contact);
            SetEmails(contactItem, source.Contact);
            SetPhonenumbers(contactItem, source.Contact);
            SetWebsites(contactItem, source.Contact);
            SetIMS(contactItem, source.Contact);
            SetDates(contactItem, source.Contact);
            SetContent(contactItem, source.Contact);
            SetPhoto(contactItem, source.Photo);

            return contactItem;
        }

        /// <summary>
        /// Sets the name.
        /// </summary>
        /// <param name="destination">The destination.</param>
        /// <param name="source">The source.</param>
        private static void SetName(Outlook.ContactItem destination, Contact source)
        {
            destination.FirstName = source.Name.GivenName;
            destination.LastName = source.Name.FamilyName;

            destination.FileAs = destination.LastNameAndFirstName;
        }

        /// <summary>
        /// Sets the addresses.
        /// </summary>
        /// <param name="destination">The destination.</param>
        /// <param name="source">The source.</param>
        private static void SetAddresses(Outlook.ContactItem destination, Contact source)
        {
            foreach (var address in source.PostalAddresses)
            {
                if (address.Rel == ContactsRelationships.IsHome)
                {
                    destination.HomeAddressCity = address.City;
                    destination.HomeAddressCountry = address.Country;
                    destination.HomeAddressPostalCode = address.Postcode;
                    destination.HomeAddressState = address.Region;
                    destination.HomeAddressStreet = address.Street;
                }
                else if (address.Rel == ContactsRelationships.IsWork)
                {
                    destination.BusinessAddressCity = address.City;
                    destination.BusinessAddressCountry = address.Country;
                    destination.BusinessAddressPostalCode = address.Postcode;
                    destination.BusinessAddressState = address.Region;
                    destination.BusinessAddressStreet = address.Street;
                }
                else if (address.Rel == ContactsRelationships.IsOther)
                {
                    destination.OtherAddressCity = address.City;
                    destination.OtherAddressCountry = address.Country;
                    destination.OtherAddressPostalCode = address.Postcode;
                    destination.OtherAddressState = address.Region;
                    destination.OtherAddressStreet = address.Street;
                }
            }
        }

        /// <summary>
        /// Sets the emails.
        /// </summary>
        /// <param name="destination">The destination.</param>
        /// <param name="source">The source.</param>
        private static void SetEmails(Outlook.ContactItem destination, Contact source)
        {
            if (source.Emails.Count > 0)
            {
                destination.Email1Address = source.Emails[0].Address;
                destination.Email1DisplayName = source.Name.FullName;
            }
            else if (source.Emails.Count > 1)
            {
                destination.Email2Address = source.Emails[1].Address;
                destination.Email2DisplayName = source.Name.FullName;
            }
            else if (source.Emails.Count > 2)
            {
                destination.Email3Address = source.Emails[2].Address;
                destination.Email3DisplayName = source.Name.FullName;
            }
        }

        /// <summary>
        /// Sets the phonenumbers.
        /// </summary>
        /// <param name="destination">The destination.</param>
        /// <param name="source">The source.</param>
        private static void SetPhonenumbers(Outlook.ContactItem destination, Contact source)
        {
            foreach (var phone in source.Phonenumbers)
            {
                if (phone.Rel == ContactsRelationships.IsHome)
                {
                    destination.HomeTelephoneNumber = phone.Value;
                }
                else if (phone.Rel == ContactsRelationships.IsWork)
                {
                    destination.BusinessTelephoneNumber = phone.Value;
                }
                else if (phone.Rel == ContactsRelationships.IsMobile)
                {
                    destination.MobileTelephoneNumber = phone.Value;
                }
                else if (phone.Rel == ContactsRelationships.IsWorkFax)
                {
                    destination.BusinessFaxNumber = phone.Value;
                }
                else if (phone.Rel == ContactsRelationships.IsHomeFax)
                {
                    destination.HomeFaxNumber = phone.Value;
                }
                else if (phone.Rel == ContactsRelationships.IsPager)
                {
                    destination.PagerNumber = phone.Value;
                }
                else if (phone.Rel == ContactsRelationships.IsSatellite)
                {
                    destination.RadioTelephoneNumber = phone.Value;
                }
                else if (phone.Rel == ContactsRelationships.IsOther)
                {
                    destination.OtherTelephoneNumber = phone.Value;
                }
                else if (phone.Rel == ContactsRelationships.IsCar)
                {
                    destination.CarTelephoneNumber = phone.Value;
                }
                else if (phone.Rel == ContactsRelationships.IsVoip)
                {
                    destination.Business2TelephoneNumber = phone.Value;
                }
            }
        }

        /// <summary>
        /// Sets the websites.
        /// </summary>
        /// <param name="destination">The destination.</param>
        /// <param name="source">The source.</param>
        private static void SetWebsites(Outlook.ContactItem destination, Contact source)
        {
            if (source.ContactEntry.Websites.Count > 0)
            {
                Website website = source.ContactEntry.Websites.FirstOrDefault(w => w.Primary);

                if (website == null)
                {
                    website = source.ContactEntry.Websites[0];
                }

                destination.WebPage = website.Href;
            }
        }

        /// <summary>
        /// Sets the IMS.
        /// </summary>
        /// <param name="destination">The destination.</param>
        /// <param name="source">The source.</param>
        private static void SetIMS(Outlook.ContactItem destination, Contact source)
        {
            string messagingAdresses = string.Empty;

            foreach (IMAddress im in source.IMs)
            {
                if (messagingAdresses.Length > 0)
                {
                    messagingAdresses += "; ";
                }

                if (!string.IsNullOrEmpty(im.Protocol))
                {
                    messagingAdresses += im.Protocol.Substring(im.Protocol.IndexOf("#") + 1) + ": " + im.Address;
                }
            }

            destination.IMAddress = messagingAdresses;
        }

        /// <summary>
        /// Sets the dates.
        /// </summary>
        /// <param name="destination">The destination.</param>
        /// <param name="source">The source.</param>
        private static void SetDates(Outlook.ContactItem destination, Contact source)
        {
            if (source.ContactEntry.Birthday != null)
            {
                destination.Birthday = DateTime.Parse(source.ContactEntry.Birthday);
            }

            if (source.ContactEntry.Events.Count > 0)
            {
                var anniversary = source.ContactEntry.Events[0];

                if (anniversary.Relation == "anniversary")
                {
                    destination.Anniversary = anniversary.When.StartTime;
                }
            }
        }

        /// <summary>
        /// Sets the content.
        /// </summary>
        /// <param name="destination">The destination.</param>
        /// <param name="source">The source.</param>
        private static void SetContent(Outlook.ContactItem destination, Contact source)
        {
            destination.Body = source.Content;
        }

        /// <summary>
        /// Sets the photo.
        /// </summary>
        /// <param name="destination">The destination.</param>
        /// <param name="photo">The photo.</param>
        private static void SetPhoto(Outlook.ContactItem destination, byte[] photo)
        {
            if (photo == null)
            {
                return;
            }

            string imagePath = Path.Combine(Path.GetTempPath(), Guid.NewGuid().ToString() + ".jpg");
            File.WriteAllBytes(imagePath, photo);

            destination.AddPicture(imagePath);

            File.Delete(imagePath);
        }
    }
}
