﻿using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Net;
using Google.Contacts;
using Google.GData.Client;
using Google.GData.Contacts;

namespace Palmmedia.GoogleSync.Synchronization
{
    /// <summary>
    /// Fetches the contacts and the photos from Google.
    /// </summary>
    internal static class ContactResolver
    {
        /// <summary>
        /// Retrieves all contacts from Google.
        /// </summary>
        /// <param name="username">The username.</param>
        /// <param name="password">The password.</param>
        /// <returns>All contacts.</returns>
        public static IEnumerable<ContactWrapper> RetrieveAll(string username, string password)
        {
            string authToken = GetAuthToken(username, password);

            var rs = new RequestSettings("GoogleSync", username, password);
            rs.AutoPaging = true; // AutoPaging results in automatic paging in order to retrieve all contacts

            var contactRequest = new ContactsRequest(rs);
            var entries = contactRequest.GetContacts().Entries.ToArray();

            var result = entries.Select(e => new ContactWrapper(e, GetPhoto(authToken, e)));
            return result.ToArray();
        }

        /// <summary>
        /// Gets the photo of the given contact.
        /// </summary>
        /// <param name="authToken">The auth token.</param>
        /// <param name="contact">The contact.</param>
        /// <returns>The photo.</returns>
        private static byte[] GetPhoto(string authToken, Contact contact)
        {
            if (contact.PhotoUri == null)
            {
                return null;
            }

            using (var client = new WebClient())
            {
                client.Headers.Add(HttpRequestHeader.Authorization, "GoogleLogin auth=" + authToken);
                try
                {
                    using (var stream = client.OpenRead(contact.PhotoUri.AbsoluteUri))
                    {
                        using (var result = new MemoryStream())
                        {
                            stream.CopyTo(result);
                            return result.ToArray();
                        }
                    }
                }
                catch (WebException)
                {
                    return null;
                }
            }
        }

        /// <summary>
        /// Gets the auth token for Google.
        /// </summary>
        /// <param name="username">The username.</param>
        /// <param name="password">The password.</param>
        /// <returns>The auth token.</returns>
        private static string GetAuthToken(string username, string password)
        {
            var contactsService = new ContactsService("GoogleSync");
            contactsService.setUserCredentials(username, password);
            string authToken = contactsService.QueryClientLoginToken();
            return authToken;
        }
    }
}
