﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.InteropServices;
using System.Threading.Tasks;
using Palmmedia.GoogleSync.Common;
using Palmmedia.GoogleSync.MessageHandling;
using Outlook = Microsoft.Office.Interop.Outlook;

namespace Palmmedia.GoogleSync.Synchronization
{
    /// <summary>
    /// Imports Google contacts to Outlook.
    /// </summary>
    internal class Synchronizer
    {
        /// <summary>
        /// The default date that is applied instead of <c>null</c>.
        /// </summary>
        private static readonly DateTime DEFAULTDATE = new DateTime(4501, 1, 1);

        /// <summary>
        /// Reference to the Outlook application.
        /// </summary>
        private readonly Microsoft.Office.Interop.Outlook.Application application;

        /// <summary>
        /// The <see cref="IMessageHandler"/>.
        /// </summary>
        private readonly IMessageHandler messageHandler;

        /// <summary>
        /// The number of expected calender events (e.g. birthdays).
        /// </summary>
        private int expectedNumberOfCalendarEvents = 0;

        /// <summary>
        /// The <see cref="CalendarMonitor"/>.
        /// </summary>
        private CalendarMonitor monitor;

        /// <summary>
        /// Initializes a new instance of the <see cref="Synchronizer"/> class.
        /// </summary>
        /// <param name="application">The Outlook application.</param>
        /// <param name="messageHandler">The <see cref="IMessageHandler"/>.</param>
        public Synchronizer(Microsoft.Office.Interop.Outlook.Application application, IMessageHandler messageHandler)
        {
            this.application = application;
            this.messageHandler = messageHandler;
        }

        /// <summary>
        /// Occurs when import is finished.
        /// </summary>
        public event EventHandler<EventArgs> Finished;

        /// <summary>
        /// Synchronizes the contacts.
        /// </summary>
        /// <param name="settings">The settings.</param>
        public void Synchronize(Settings settings)
        {
            var contactsTask = Task<IEnumerable<ContactWrapper>>.Factory.StartNew(
                () => ContactResolver.RetrieveAll(settings.Username, settings.Password));

            contactsTask.ContinueWith(
                contacts =>
                {
                    if (contacts.Exception != null)
                    {
                        this.messageHandler.ShowMessage("Error during resolving contacts: " + contacts.Exception.GetFullMessage());
                    }
                    else
                    {
                        int deletedContacts = 0;

                        if (settings.DeleteExistingContacts)
                        {
                            try
                            {
                                deletedContacts = this.DeleteExistingContacts();
                            }
                            catch (Exception ex)
                            {
                                this.messageHandler.ShowMessage("Error during deleting existing contacts: " + ex.Message);
                            }
                        }

                        int successfulImports = 0;

                        if (!settings.CreateEvents)
                        {
                            this.monitor = new CalendarMonitor(this.application.ActiveExplorer());
                            this.monitor.AppointmentAdded += this.EventAdded;
                        }

                        foreach (var contact in contacts.Result)
                        {
                            try
                            {
                                ImportContact(contact);

                                successfulImports++;
                            }
                            catch (Exception ex)
                            {
                                this.messageHandler.ShowMessage("Error during importing contact: " + ex.Message);
                            }
                        }

                        this.messageHandler.ShowMessage(string.Format(
                            "Successfully imported {0} of {1} contacts.{2}{3} existing contacts have been deleted.",
                            successfulImports,
                            contacts.Result.Count(),
                            System.Environment.NewLine,
                            deletedContacts));
                        this.OnFinished(EventArgs.Empty);
                    }
                },
                TaskScheduler.FromCurrentSynchronizationContext());
        }

        /// <summary>
        /// Imports the given contact.
        /// </summary>
        /// <param name="contact">The contact.</param>
        private void ImportContact(ContactWrapper contact)
        {
            var outlookContact = ContactBuilder.CreateContact(this.application, contact);

            if (!outlookContact.Birthday.Equals(DEFAULTDATE))
            {
                this.expectedNumberOfCalendarEvents++;
            }

            if (!outlookContact.Anniversary.Equals(DEFAULTDATE))
            {
                this.expectedNumberOfCalendarEvents++;
            }

            outlookContact.Save();

            Marshal.ReleaseComObject(outlookContact);
            outlookContact = null;
        }

        /// <summary>
        /// Deletes all existing contacts.
        /// </summary>
        /// <returns>The number of deleted contacts.</returns>
        private int DeleteExistingContacts()
        {
            var mapiFolder = this.application.ActiveExplorer().Session.GetDefaultFolder(Outlook.OlDefaultFolders.olFolderContacts);

            var items = new List<object>();

            var contacts = mapiFolder.Items;
            foreach (var item in contacts)
            {
                items.Add(item);
            }

            int deletedContacts = 0;

            foreach (var item in items)
            {
                var contact = item as Outlook.ContactItem;

                if (contact != null)
                {
                    contact.Delete();
                    deletedContacts++;

                    Marshal.ReleaseComObject(contact);
                    contact = null;
                }
            }

            Marshal.ReleaseComObject(contacts);
            contacts = null;

            return deletedContacts;
        }

        /// <summary>
        /// Deletes a new created event.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="item">The <see cref="GoogleSync.Common.EventArgs&lt;Microsoft.Office.Interop.Outlook.AppointmentItem&gt;"/> instance containing the event data.</param>
        private void EventAdded(object sender, Common.EventArgs<Outlook.AppointmentItem> item)
        {
            item.Value.Delete();
            this.expectedNumberOfCalendarEvents--;

            // If all events have been added, the eventhandler can be removed
            if (this.expectedNumberOfCalendarEvents == 0)
            {
                this.monitor.AppointmentAdded -= this.EventAdded;
            }
        }

        /// <summary>
        /// Raises the <see cref="E:Finished"/> event.
        /// </summary>
        /// <param name="eventArgs">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void OnFinished(EventArgs eventArgs)
        {
            if (this.Finished != null)
            {
                this.Finished(this, eventArgs);
            }
        }
    }
}
